#!/usr/bin/env ruby
#
# gpv2nc
# Copyright (C) by GFD-Dennou Club, 2002.  All rights reserved.

def usage
   return <<-EOS

   #{$0}: Converts GPV files into a NetCDF file

   Usage:
      % #{$0} [-ihd] [-o output_file] gpv_files.. 

      -o output_file : output file name (if omitted, determined internally)
      -h : help (to show this message)
      -d : debug mode (does extra printing)
      -i : shows info on the input file (does not dump a file, but creates
           a temporarlly file -- if you do not have a write permission of the
           current directory, specify one you have it by the envronmental
           variable TMPDIR. Example: % export TMPDIR=/tmp).
   EOS
end
#############################################################
require 'numru/gpv/bugaifile.rb'
require "getopts"
include NumRu

class Array
  def add_time(sec)
    hour = (sec/3600.0).round
    self.collect{|x| x+hour}
  end
end

if( getopts('ihd',"o:gpv.nc") && ! $OPT_h )
   ofilename = $OPT_o
   show_info = $OPT_i
   $DEBUG = $OPT_d
else
   raise usage
end

outfmt = 'netcdf'

GPVID = %w(institution source grid average)
gpvid = Hash::new
planetype = nil
basetime = []
basetime0 = nil
institution = nil
source = nil

plane = []
time_since_b0 = []
xlist = []
ylist = []
elems = []

ARGV.each { |fnam|
    fp = BugaiFile::new(fnam, "r")
    fp.each { |rec|
        bull = rec.decode
        raise "currently only DGRB is supported" unless bull.is_a? DGRB
        bull.each { |sec|
            raise "currently only DGRB/GPV is supported" \
		unless sec.is_a? DGRB::StdSection
	    if planetype.nil? then
		print "prescanning ...\n"; $defout.flush
		GPVID.each {|k| gpvid[k] = sec.send(k) }
                institution = sec.institution.to_s
                source = sec.source.to_s
		planetype = sec.planetype
                basetime0 = sec.basetime
                basetime.push( basetime0 )
	    else
		GPVID.each {|k|
		    raise "#{k} mismatch (#{gpvid[k]} != #{sec.send(k)})" \
			if not gpvid[k] == sec.send(k)
		}
		raise "planetype mismatch (#{planetype}, #{sec.planetype})" \
		    unless planetype.compatible? sec.planetype
	    end
	    plane = (plane.push sec.plane1).uniq
	    basetime = (basetime.push sec.basetime).uniq
#	    validtime = (validtime + sec.validtime_list).uniq
	    time_since_b0 = ( time_since_b0 + 
                  sec.validtime_list.add_time(sec.basetime-basetime0) ).uniq
	    xlist = (xlist + sec.xlist).uniq
	    ylist = (ylist + sec.ylist).uniq
            elems = (elems.push sec.format.to_i).uniq
        }
    }
    fp.close
}

p "plane list", plane if $DEBUG
p "basetime list", basetime if $DEBUG
p "time list", time_since_b0 if $DEBUG
p "i grid number", xlist if $DEBUG
p "j grid number", ylist if $DEBUG
p "element list", elems if $DEBUG

if outfmt == 'netcdf' then

    require 'numru/netcdf'
    if (show_info)
        output = NumRu::NetCDF::create_tmp
    else
        output = NumRu::NetCDF::create(ofilename)
      print "file #{ofilename} created ...\n"; $defout.flush
    end

    output.put_att('Conventions', 'CF-1.0')
    output.put_att('source', source)
    output.put_att('institution', institution)
    output.put_att('history', 
           "#{Time.now.strftime("%Y-%m-%d %H:%M")}: #{File.basename($0)} " +
           ARGV.join(" "))

    dimx = output.def_dim('lon', xlist.size)
    dimy = output.def_dim('lat', ylist.size)
    dimz = output.def_dim('p', plane.size) if not plane.first.nil?
    dimt = output.def_dim('time', time_since_b0.size)
    dimrt = output.def_dim('ref_time', basetime.size)

    output.def_var('lon', "sfloat", [dimx])
    output.var('lon').put_att('long_name', 'longitude')
    output.var('lon').put_att('standard_name', 'longitude')
    output.var('lon').put_att('units', 'degrees_east')
    output.def_var('lat', "sfloat", [dimy])
    output.var('lat').put_att('long_name', 'latitude')
    output.var('lat').put_att('standard_name', 'latitude')
    output.var('lat').put_att('units', 'degrees_north')
    if not plane.first.nil?
        output.def_var('p', "sfloat", [dimz]) 
        output.var('p').put_att('long_name', 'pressure level')
        output.var('p').put_att('standard_name', 'air_pressure')
        output.var('p').put_att('units', planetype.units.to_s)
    end
    output.def_var('time', "sfloat", [dimt])
    output.var('time').put_att('long_name', 'time')
    output.var('time').put_att('standard_name', 'time')
    units = "hours since " + basetime0.getgm.strftime("%Y-%m-%d %H:%M:%S+00:00")
    output.var('time').put_att('units', units)
    output.def_var('ref_time', "sfloat", [dimrt])
    output.var('ref_time').put_att('long_name', 'forecaset reference time')
    output.var('ref_time').put_att('standard_name', 'forecaset_reference_time')
    output.var('ref_time').put_att('units', units)

    elems.each { |ie|
        e = DGRB::Parameter::new(ie)
        varname = e.altname
        if ! (plane.first.nil?)
            output.def_var(varname, "sfloat", [dimx, dimy, dimz, dimt])
	else
            output.def_var(varname, "sfloat", [dimx, dimy, dimt])
	end
        output.var(varname).put_att('long_name', e.alttitle)
        (sn = e.cf_standard_name) &&
                 output.var(varname).put_att('standard_name', sn)
        output.var(varname).put_att('units', e.units)
    }   

    output.enddef

    output.var('lon').put gpvid['grid'].lon_list(xlist)
    output.var('lat').put gpvid['grid'].lat_list(ylist)
    if not plane.first.nil?
        output.var('p').put plane
    end
    output.var('time').put(time_since_b0.collect(){|e| e.to_i })
    output.var('ref_time').put(basetime.collect{|bt| ((bt-basetime0)/3600).round})

    if (show_info)
        output.close
        p output.path if $DEBUG
        print `ncdump -c #{output.path}`
        exit
    end

    ARGV.each { |fnam|
	fp = BugaiFile::new(fnam, "r")
	fp.each { |rec|
	    bull = rec.decode
	    raise "currently only DGRB is supported" unless bull.is_a? DGRB
	    bull.each { |sec|
		tlist = sec.validtime_list.add_time(sec.basetime-basetime0)
                ip = plane.index sec.plane1
		iv = time_since_b0.index tlist.first
		ix = xlist.index sec.xlist.first
		iy = ylist.index sec.ylist.first
                nv = sec.format.altname
                kx = xlist.index sec.xlist.last
                ky = ylist.index sec.ylist.last
		kv = time_since_b0.index tlist.last
		if (tlist.size > 1) then
		    sv = (kv - iv) / (tlist.size - 1)
		else
		    sv = 1
		end
		#p [iv, ip, iy, ix]
		# "->", kv, ip, ky, kx, "nv", \
		#    ">>", sv, ":", sec.get_vals.size ]
	        if ! (plane.first.nil?)
	            output.var(nv).put(sec.get_vals, \
		        "start" => [ix, iy, ip, iv], \
                        "end" => [kx, ky, ip, kv],
		        "stride" => [1, 1, 1, sv])
		else
	            output.var(nv).put(sec.get_vals, \
		        "start" => [ix, iy, iv], \
                        "end" => [kx, ky, kv],
		        "stride" => [1, 1, sv])
		end
            }
        }
    }

    output.close

end
